<?php
namespace EDBB_Plugin;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles admin logic
 *
 * @since 1.0.0
 */
final class Admin {

	/**
	 * Holds the current class object instance.
	 *
	 * @since 1.0.0
	 * @var Admin
	 */
	public static $instance;

	/**
	 * Holds the settings page slug.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	public $settings_page;

	/**
	 * Holds the settings page title.
	 *
	 * @since 1.0.0
	 * @var string
	 */
	public $settings_title;

	/**
	 * Holds the plugin settings.
	 *
	 * @since 1.0.0
	 * @var array
	 */
	public $settings;

	/**
     * Holds the user roles.
     *
     * @since 1.0.0
     * @var array
     */
    public $roles;

    /**
     * Holds the current user role.
     *
     * @since 1.0.0
     * @var string
     */
    public $current_role;

	/**
	 * Initializes the admin settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function __construct() {
		// Bail if not in the admin area.
		if ( ! is_admin() ) {
			return;
		}

		// Initialize properties.
		$this->settings_page  = 'easydash-bricks-settings';
		$this->settings_title = esc_html__( 'EasyDash for Bricks', 'easydash-bricks' );
		$this->settings       = $this->get_settings();

		// Add actions.
		add_action( 'admin_menu', array( $this, 'admin_menu' ), 1000 );
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'edbb_admin_enqueue_scripts' ) );
	}

	/**
	 * Initializes admin logic and settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function admin_init() {
		register_setting( 'edbb_options_group', 'edbb_dashboards', [ $this, 'sanitize_dashboards' ] );
		register_setting( 'edbb_options_group', 'edbb_hide_from_subsites', [ $this, 'sanitize_hide_from_subsites' ] );

		$this->setup_admin_notices();

		// Only apply the logic on the admin dashboard (index.php).
		global $pagenow;
		if ( 'index.php' !== $pagenow ) {
			return;
		}

		$settings     = $this->settings;
		$template_set = false;

		// Return early if there are no settings.
		if ( empty( $settings ) ) {
			return;
		}

		// Check if any template is set.
		foreach ( $settings as $value ) {
			$template_id  = $value['template'];
			$template_set = true;
			break;
		}

		// If a template is set, manage the welcome panel.
		if ( $template_set ) {
			remove_action( 'welcome_panel', 'wp_welcome_panel' );
			add_action( 'welcome_panel', array( $this, 'welcome_panel' ) );

			$bricks_setup = new \Bricks\Setup();
			$bricks_setup->enqueue_scripts();

			// For users without theme editing capabilities, display a custom welcome panel.
			if ( ! current_user_can( 'edit_theme_options' ) ) {
				add_action( 'admin_notices', array( $this, 'welcome_panel' ) );
				$this->enqueue_welcome_panel_scripts();
			}
		}
	}

	public function sanitize_dashboards( $input ) {
		if ( ! is_array( $input ) ) {
			return [];
		}

		foreach ( $input as $key => $value ) {
			$input[$key]['template'] = absint( $value['template'] ); // Integer
			$input[$key]['condition_type'] = sanitize_text_field( $value['condition_type'] ); // String
			$input[$key]['user_role'] = sanitize_text_field( $value['user_role'] ); // String
			$input[$key]['user_id'] = absint( $value['user_id'] ); // Integer
			$input[$key]['dismissible'] = isset( $value['dismissible'] ) ? (bool) $value['dismissible'] : false; // Boolean (converted to true/false)
		}

		return $input;
	}	

	public function sanitize_hide_from_subsites( $input ) {
		return (bool) $input;
	}

	/**
	 * Renders the welcome panel.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function welcome_panel() {
		include EDBB_DIR . 'includes/welcome-panel.php';
	}

	/**
	 * Enqueues the welcome panel scripts for users without theme editing capabilities.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function enqueue_welcome_panel_scripts() {
		wp_enqueue_script(
			'edbb-welcome-panel',
			EDBB_URL . 'assets/js/welcome-panel.js',
			array( 'jquery' ),
			EDBB_VER,
			true
		);
	}

	/**
	 * Enqueues admin scripts and styles.
	 *
	 * @since 1.0.0
	 * @param string $hook The current admin page hook.
	 * @return void
	 */
	public function edbb_admin_enqueue_scripts( $hook ) {
		// Only enqueue scripts on the EasyDash settings page.
		if ( 'bricks_page_easydash-bricks-settings' !== $hook ) {
			return;
		}

		wp_enqueue_script(
			'edbb-select2',
			EDBB_URL . 'assets/js/select2.min.js',
			array( 'jquery' ),
			'4.0.13',
			true
		);

		wp_enqueue_script( 'jquery-ui-sortable' );

		wp_enqueue_script(
			'edbb-admin',
			EDBB_URL . 'assets/js/admin.js',
			array( 'jquery', 'jquery-ui-sortable' ),
			EDBB_VER,
			true
		);

		wp_localize_script( 'edbb-admin', 'edbbData', array(
			'isMultisite' => is_multisite(),
		));

		wp_enqueue_style(
			'edbb-select2',
			EDBB_URL . 'assets/css/select2.min.css',
			array(),
			'4.0.13'
		);

		wp_enqueue_style(
			'edbb-admin',
			EDBB_URL . 'assets/css/admin.css',
			array(),
			EDBB_VER
		);
	}

	/**
	 * Adds the Dashboard Welcome to the admin menu.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function admin_menu() {
		global $wp_roles;

		$this->roles        = $wp_roles->get_names();
		$this->current_role = $this->get_user_role();

		// Check if settings are hidden for subsites in a multisite setup.
		if ( is_multisite() ) {
			$hide_settings = get_blog_option( 1, 'edbb_hide_from_subsites' );
			if ( $hide_settings && get_current_blog_id() !== 1 ) {
				return;
			}
		}

		// Add settings page for users with proper capabilities.
		if ( current_user_can( 'manage_options' ) ) {
			add_submenu_page(
				'bricks',
				esc_html__( 'EasyDash', 'easydash-bricks' ),
				esc_html__( 'EasyDash', 'easydash-bricks' ),
				'manage_options',
				$this->settings_page,
				array( $this, 'render_settings' )
			);
		}
	}

	/**
	 * Renders the settings page content.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function render_settings() {
		$title     = $this->settings_title;
		$roles     = $this->roles;
		$users     = \Bricks\Helpers::get_users_options( array(), true );
		$templates = $this->get_templates();
		$settings  = $this->get_settings();

		include EDBB_DIR . 'includes/admin-settings.php';
	}

	/**
	 * Renders the Bricks template in the welcome panel.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function render_template() {
		$settings    = $this->get_settings();
		$template_id = null;

		if ( empty( $settings ) ) {
			return;
		}

		foreach ( $settings as $key => $value ) {
			$condition_type = $value['condition_type'];

			if ( 'user' === $condition_type && (int) $value['user_id'] === get_current_user_id() ) {
				$template_id = (int) $value['template'];
				$dismissible = isset( $value['dismissible'] ) ? true : false;
				break;
			}

			if ( 'user_role' === $condition_type && $value['user_role'] === $this->current_role ) {
				if ( ! $template_id ) {
					$template_id = (int) $value['template'];
					$dismissible = isset( $value['dismissible'] ) ? true : false;
				}
			}
		}

		if ( $template_id ) {
			$site_id      = isset( $value['site'] ) ? intval( $value['site'] ) : '';
			$is_multisite = is_multisite();

			// Enqueue dashboard styles.
			wp_enqueue_style(
				'edbb-dashboard-css',
				EDBB_URL . 'assets/css/dashboard.css',
				array(),
				EDBB_VER
			);

			if ( ! $dismissible ) {
				wp_add_inline_style( 'edbb-dashboard-css', '.welcome-panel .welcome-panel-close { display: none; }' );
			}

			// Enqueue Bricks frontend styles.
			$bricks = new \Bricks\Frontend();
			$bricks->enqueue_inline_css();

			// Get and generate inline CSS.
			$elements   = get_post_meta( $template_id, BRICKS_DB_PAGE_CONTENT, true );
			$inline_css = \Bricks\Templates::generate_inline_css( $template_id, $elements );

			// Add inline CSS for the frontend.
			wp_add_inline_style( 'bricks-frontend-inline', esc_html( $inline_css ) );

			// Switch to blog for multisite.
			if ( ! empty( $site_id ) && $is_multisite ) {
				switch_to_blog( $site_id );
			}

			// Output the template using a shortcode.
			echo do_shortcode( '[bricks_template id="' . esc_attr( $template_id ) . '" ]' );

			// Restore the blog if switched.
			if ( ! empty( $site_id ) && $is_multisite ) {
				restore_current_blog();
			}
		}
	}

	/**
	 * Get Bricks saved templates.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	private function get_templates() {
		$args = array(
			'post_type'      => BRICKS_DB_TEMPLATE_SLUG,
			'posts_per_page' => -1,
			'meta_query'     => array(
				array(
					'key'   => BRICKS_DB_TEMPLATE_TYPE,
					'value' => array( 'section', 'content' ),
				),
			),
			'post_status'    => 'publish',
		);

		$templates = get_posts( $args );

		// Multisite support.
		if ( is_multisite() ) {
			$blog_id = get_current_blog_id();

			if ( $blog_id !== 1 ) {
				switch_to_blog( 1 );
				$main_posts = get_posts( $args );

				// Add site_id to each main site post.
				foreach ( $main_posts as $main_post ) {
					$main_post->site_id = 1;
				}

				$templates = array_merge( $templates, $main_posts );
				restore_current_blog();
			} else {
				// Add site_id to each current site post.
				foreach ( $templates as $template ) {
					$template->site_id = 1;
				}
			}
		}

		$data = array();
		if ( ! empty( $templates ) && ! is_wp_error( $templates ) ) {
			foreach ( $templates as $post ) {
				$data[ $post->ID ] = array(
					'title' => esc_html( $post->post_title ),
					'site'  => isset( $post->site_id ) ? intval( $post->site_id ) : null,
				);
			}
		}

		return $data;
	}

	/**
	 * Get the current user's role.
	 *
	 * @since 1.0.0
	 * @return string|null The user's role or null if no role is found.
	 */
	private function get_user_role() {
		$user  = wp_get_current_user();
		$roles = $user->roles;

		return ! empty( $roles ) ? $roles[0] : null;
	}

	/**
	 * Get the plugin settings.
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function get_settings() {
		$key = 'edbb_dashboards';

		if ( is_multisite() ) {
			$settings = get_option( $key, false );
			$settings = ! $settings ? get_blog_option( 1, $key ) : $settings;
		} else {
			$settings = get_option( $key, array() );
		}

		if ( ! is_array( $settings ) ) {
			$settings = array();
		}

		$this->settings = $settings;
		return $settings;
	}

	public function setup_admin_notices() {
		add_action('admin_notices', function() {
			// Check if settings were saved successfully
			if ( isset( $_GET['settings-updated'] ) && $_GET['settings-updated'] ) {
				echo '<div class="notice notice-success is-dismissible"><p>' . esc_html__('Settings saved.', 'easydash-bricks') . '</p></div>';
			}
		});
	}	

	/**
	 * Get the class instance (singleton pattern).
	 *
	 * @since 1.0.0
	 * @return object
	 */
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof EDBB_Plugin\Admin ) ) {
			self::$instance = new Admin();
		}

		return self::$instance;
	}
}
